from django.conf import settings


def get_talk_choices():
    """
    A list of choice types.
    """

    return getattr(
        settings,
        "UNCONFERENCE_TALK_CHOICES",
        (
            (1, "Talk"),
            (2, "Discussion"),
            (3, "Hands-On"),
        )
    )



from django.contrib.auth import get_user_model, login
import random, functools


def ensure_guest_login(func):
  @functools.wraps(func)
  def wrapped(request, *args, **kwargs):
    if not request.user.is_authenticated:
        User = get_user_model()
        email = f'guest+{random.random()}@example.com'
        user = User.objects.create(email=email)
        backend = 'django.contrib.auth.backends.ModelBackend'
        login(request, user, backend=backend)
    return func(request, *args, **kwargs)
  return wrapped



# Generated by Django 4.2.6 on 2024-10-01 20:07

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("unconference", "0002_usereventdata"),
    ]

    operations = [
        migrations.CreateModel(
            name="Location",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.CharField(max_length=128)),
                ("geometry", models.JSONField(default=dict)),
            ],
        ),
        migrations.RemoveConstraint(
            model_name="room",
            name="unique_room",
        ),
        migrations.RemoveField(
            model_name="room",
            name="unconference_event",
        ),
        migrations.AddField(
            model_name="room",
            name="geometry",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="session",
            name="data",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="session",
            name="unconference_event",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.PROTECT,
                to="unconference.unconferenceevent",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="session",
            name="users",
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name="session",
            name="schedule_time",
            field=models.ForeignKey(
                blank=True,
                limit_choices_to={"allow_sessions": True},
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="unconference.scheduletime",
            ),
        ),
        migrations.AddField(
            model_name="room",
            name="location",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="unconference.location",
            ),
        ),
        migrations.AddField(
            model_name="unconferenceevent",
            name="location",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="unconference.location",
            ),
        ),
    ]



from django.contrib import admin
from django.forms.widgets import TextInput
from django.db.models import TextField

from .models import UnConferenceEvent, ScheduleTime, Room, Session, UserEventData


class TextInputModelAdmin(admin.ModelAdmin):
    formfield_overrides = {
        TextField: {'widget': TextInput},
    }


@admin.register(UnConferenceEvent)
class UnConferenceEventAdmin(TextInputModelAdmin):
    list_display = ("title", "start", "end", "active")
    search_fields = ("title",)
    ordering = ("-active", "-start")
    fields = ("title", "start", "end", "active")
    list_filter = ("active",)


@admin.register(ScheduleTime)
class ScheduleTimeAdmin(TextInputModelAdmin):
    list_display = ("title", "start", "end", "allow_sessions", "unconference_event")
    search_fields = ("title", "unconference_event")
    ordering = ("title", "unconference_event")
    fields = ("title", "start", "end", "allow_sessions", "unconference_event")
    list_filter = ("allow_sessions", "unconference_event")


@admin.register(Room)
class RoomAdmin(TextInputModelAdmin):
    list_display = ("title", "capacity", "location")
    search_fields = ("title",)
    ordering = ("title", "location")
    fields = ("title", "capacity", "description", "location")
    list_filter = ("location",)


@admin.register(Session)
class SessionAdmin(admin.ModelAdmin):
    list_display = ("title", "leaders", "schedule_time", "room")
    search_fields = ("title", "description")
    ordering = ("schedule_time", "room")
    fields = ("schedule_time", "room", "leaders", "title", "description", "session_type", "users")
    filter_horizontal = ('users',)
    list_filter = ("schedule_time__unconference_event", "session_type")
    def get_field_queryset(self, db, db_field, request):
        if db_field.name == 'users':

            return db_field.remote_field.model._default_manager.exclude(
                email__endswith="example.com",
            )

        super().get_field_queryset(db, db_field, request)


@admin.register(UserEventData)
class UserEventDataAdmin(admin.ModelAdmin):
    """
    Readonly admin for UserEventData
    """
    list_display = ("user", "unconference_event")
    readonly_fields = ("user", "unconference_event", "data")

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False


from django.conf import settings
from django.db import models

from .settings import get_talk_choices


class UnConferenceEvent(models.Model):
    """
    Model to store unconference events.
    """

    title = models.TextField(null=False, blank=False)
    start = models.DateTimeField(null=False)
    end = models.DateTimeField(null=False)
    active = models.BooleanField(default=False)
    location = models.ForeignKey('Location', on_delete=models.SET_NULL, null=True, blank=True)

    class Meta:
        verbose_name = "UnConference event"

    def __str__(self):
        return self.title


class Location(models.Model):
    """
    The building where a conference occurs
    """
    title = models.CharField(max_length=128)
    geometry = models.JSONField(default=dict)

    def __str__(self):
        return self.title


def default_data():
    return {
        'votes': {},
        'attendedance': {},
    }


class UserEventData(models.Model):
    """
    Model to store a users interactions (vodting, attendance, etc) with an event.
    The structure of the data column may vary from conference to conference.
    """
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    unconference_event = models.ForeignKey(
        UnConferenceEvent,
        on_delete=models.PROTECT,
    )
    data = models.JSONField(default=default_data)


class ScheduleTime(models.Model):
    """
    Timeslots for the event's schedule; allow_sessions should be set to True for any
    timeslots where sessions can be schedule, and False for timeslots like lunch break.
    """

    unconference_event = models.ForeignKey(
        UnConferenceEvent,
        on_delete=models.PROTECT,
    )
    title = models.TextField(null=False, blank=False)
    start = models.DateTimeField(null=False)
    end = models.DateTimeField(null=False)
    allow_sessions = models.BooleanField(default=False)

    class Meta:
        constraints = [
            models.UniqueConstraint(
                fields=["unconference_event", "title"], name="unique_schedule_time"
            ),
        ]

    def __str__(self):
        return f"{self.title} ({self.start:%H:%M %p} - {self.end:%H:%M %p})"


class Room(models.Model):
    """
    Rooms where sessions can take place.
    """
    title = models.TextField(null=False, blank=False)
    capacity = models.PositiveSmallIntegerField(default=0)
    description = models.TextField(null=True, blank=True)
    location = models.ForeignKey(
        'Location',
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    geometry = models.JSONField(default=dict)

    def __str__(self):
        return f'{self.title}@{self.location}'


TALK_CHOICES = get_talk_choices()


class Session(models.Model):
    talk_choices = {}
    for t in get_talk_choices():
        talk_choices[t[0]] = t[1]

    leaders = models.TextField(null=True, blank=True)
    users = models.ManyToManyField(
        settings.AUTH_USER_MODEL,
    )
    schedule_time = models.ForeignKey(
        ScheduleTime,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        limit_choices_to={"allow_sessions": True},
    )
    room = models.ForeignKey(
        Room,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
    )
    title = models.TextField(null=False, blank=False)
    description = models.TextField(null=True, blank=True)
    session_type = models.PositiveSmallIntegerField(
        choices=get_talk_choices(),
        default=1,
        help_text="The type of session.",
    )
    data = models.JSONField(default=dict)
    unconference_event = models.ForeignKey(
        UnConferenceEvent,
        on_delete=models.PROTECT,
    )

    @property
    def leaders_info(self):
        return [
            {
                'id': user.id,
                'full_name': user.full_name,
            }
            for user in self.users.all()
        ]

    @property
    def type(self):
        return dict(TALK_CHOICES).get(self.session_type, 'UNKNOWN')

    class Meta:
        constraints = [
            models.UniqueConstraint(
                fields=["schedule_time", "room"], name="unique_booking"
            ),
        ]

    def __str__(self):
        return f"{self.title} ({self.schedule_time})"



from django.urls import path, re_path

from . import views


urlpatterns = [
    path("table/", views.HomeView.as_view()),
    path("api/event/<int:event_id>/", views.event),
    path("api/session/<int:session_id>/", views.session),
    path("api/swap-sessions/", views.swap_sessions),
    path("api/event/", views.events),
    path("api/user-event-data/<int:event_id>/", views.user_event_data),
    path("api/user.json", views.user_json),
    path("", views.index),
    re_path("^(help|settings|vote|session|map)", views.index),
]



# Generated by Django 4.2.6 on 2023-11-10 19:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Room",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.TextField()),
                ("capacity", models.PositiveSmallIntegerField(default=0)),
                ("description", models.TextField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="ScheduleTime",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.TextField()),
                ("start", models.DateTimeField()),
                ("end", models.DateTimeField()),
                ("allow_sessions", models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name="UnConferenceEvent",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("title", models.TextField()),
                ("start", models.DateTimeField()),
                ("end", models.DateTimeField()),
                ("active", models.BooleanField(default=False)),
            ],
            options={
                "verbose_name": "UnConference event",
            },
        ),
        migrations.CreateModel(
            name="Session",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("leaders", models.TextField(blank=True, null=True)),
                ("title", models.TextField()),
                ("description", models.TextField(blank=True, null=True)),
                (
                    "session_type",
                    models.PositiveSmallIntegerField(
                        choices=[(1, "Talk"), (2, "Discussion"), (3, "Hands-On")],
                        default=1,
                        help_text="The type of session.",
                    ),
                ),
                (
                    "room",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="unconference.room",
                    ),
                ),
                (
                    "schedule_time",
                    models.ForeignKey(
                        limit_choices_to={"allow_sessions": True},
                        on_delete=django.db.models.deletion.PROTECT,
                        to="unconference.scheduletime",
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="scheduletime",
            name="unconference_event",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to="unconference.unconferenceevent",
            ),
        ),
        migrations.AddField(
            model_name="room",
            name="unconference_event",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.PROTECT,
                to="unconference.unconferenceevent",
            ),
        ),
        migrations.AddConstraint(
            model_name="session",
            constraint=models.UniqueConstraint(
                fields=("schedule_time", "room"), name="unique_booking"
            ),
        ),
        migrations.AddConstraint(
            model_name="scheduletime",
            constraint=models.UniqueConstraint(
                fields=("unconference_event", "title"), name="unique_schedule_time"
            ),
        ),
        migrations.AddConstraint(
            model_name="room",
            constraint=models.UniqueConstraint(
                fields=("unconference_event", "title"), name="unique_room"
            ),
        ),
    ]



# Generated by Django 4.2.6 on 2024-08-18 23:30

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import unconference.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("unconference", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserEventData",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("data", models.JSONField(default=unconference.models.default_data)),
                (
                    "unconference_event",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="unconference.unconferenceevent",
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



from django.contrib.staticfiles import finders
from django.http import JsonResponse
from django.shortcuts import render, get_object_or_404
from django.views.decorators.csrf import ensure_csrf_cookie
from django.views.generic import TemplateView
from django.views.static import serve
import json
import os

from .models import UnConferenceEvent, ScheduleTime, Room, Session, UserEventData
from .decorators import ensure_guest_login

def serialize_one(entity, fields):
    return {
        field: getattr(entity, field)
        for field in fields
    }


def serialize(entities, fields):
    return [
        {
            field: getattr(entity, field)
            for field in fields
        }
        for entity in entities
    ]


@ensure_guest_login
def user_event_data(request, event_id):
    user_event_data, _new = UserEventData.objects.get_or_create(
        user=request.user,
        unconference_event_id=event_id,
    )
    if request.method == 'PUT':
        user_event_data.data = json.loads(request.body.decode("utf-8") or "{}")
        user_event_data.save()
    return JsonResponse({**user_event_data.data, 'id': user_event_data.id})


def swap_sessions(request):
    if not request.user.is_superuser:
        return JsonResponse({'error': 'Not allowed' }, status=403)
    data = json.loads(request.body.decode("utf-8") or "{}")
    session_1 = Session.objects.filter(id=data['session_1'].get('id')).first()
    session_2 = Session.objects.filter(id=data['session_2'].get('id')).first()

    if session_1 and session_2:
        session_1.room_id = None
        session_1.schedule_time_id = None
        session_1.save()

    if session_2:
        session_2.schedule_time_id = data['session_1']['time_id']
        session_2.room_id = data['session_1']['room_id']
        session_2.save()

    if session_1:
        session_1.schedule_time_id = data['session_2']['time_id']
        session_1.room_id = data['session_2']['room_id']
        session_1.save()

    return JsonResponse({})


def session(request, session_id):
    data = json.loads(request.body.decode("utf-8") or "{}")
    session = get_object_or_404(Session, id=session_id)
    allowed_user_ids = [u.id for u in session.users.all()]
    if not request.user.id in allowed_user_ids:
        return JsonResponse({'error': 'Not allowed' }, status=403)
    session.title = data['title']
    session.description = data['description']
    session.data['links'] = data['links']
    session.save()
    return JsonResponse({})


@ensure_guest_login
def user_json(request):
    fields = ['id', 'full_name', 'short_name', 'email', 'is_superuser', 'is_staff']
    return JsonResponse(serialize_one(request.user, fields))


def events(request):
    events = UnConferenceEvent.objects.filter(active=True)
    return JsonResponse({
        'events': serialize(events, ['id', 'title', 'start', 'end', 'active'])
    })

def event(request, event_id):
    event = get_object_or_404(UnConferenceEvent, id=event_id)
    location = event.location
    times = ScheduleTime.objects.filter(unconference_event_id=event)
    rooms = location.room_set.all()
    sessions = Session.objects.filter(unconference_event=event).prefetch_related('users')
    data = serialize_one(event, ['id', 'title', 'start', 'end', 'active'])
    data.update({
        'times': serialize(times, ['id', 'title', 'start', 'end', 'allow_sessions']),
        'rooms': serialize(rooms, ['id', 'title', 'capacity', 'description', 'geometry']),
        'location': location and serialize_one(location, ['id', 'title', 'geometry']),
        'sessions': serialize(
            sessions,
            ['id', 'leaders', 'schedule_time_id', 'room_id', 'title', 'description', 'type', 'leaders_info', 'data']
        )
    })
    return JsonResponse(data)


class HomeView(TemplateView):
    template_name = "unconference/home.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)

        if self.request.GET.get("uid", None):
            unconference_event = UnConferenceEvent.objects.filter(
                id=self.request.GET["uid"],
            )
        else:
            unconference_event = UnConferenceEvent.objects.filter(active=True)

        if unconference_event.count() == 1:
            self.template_name = "unconference/schedule.html"
            context["event"] = unconference_event[0]

            schedule = {}
            schedule_times = ScheduleTime.objects.filter(
                unconference_event=context["event"],
            ).order_by(
                "start",
            )
            rooms = Room.objects.filter(
                location=unconference_event[0].location,
            ).order_by(
                "title",
            )
            sessions = Session.objects.filter(
                schedule_time__unconference_event=context["event"],
                schedule_time__isnull=False,
                room__isnull=False,
            ).select_related('room')
            schedule = {
                r.title: {
                    s.title: {} for s in schedule_times
                } for r in rooms
            }
            for s in sessions:
                schedule[s.room.title][s.schedule_time.title] = {
                    "id": s.id,
                    "leaders": s.leaders,
                    "title": s.title,
                    "description": s.description,
                    "session_type": Session.talk_choices[s.session_type],
                }
            context["schedule_times"] = [{
                "id": time.id,
                "title": time.title,
                "start": time.start,
                "sessions": sessions.filter(schedule_time=time),
            } for time in schedule_times]

            context["sessions"] = sessions
            context["schedule"] = schedule
        else:
            context["events"] = unconference_event

        return context


@ensure_csrf_cookie
def index(request, *args, **kwargs):
    path = kwargs.get('path', finders.find('index.html'))
    return serve(
        request,
        os.path.basename(path),
        os.path.dirname(path)
    )
    return response